package org.processmining.earthmoversstochasticconformancechecking.helperclasses;

import java.util.Map;

import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.model.StochasticTransition;
import org.processmining.models.graphbased.directed.petrinet.StochasticNet;
import org.processmining.models.graphbased.directed.petrinet.elements.TimedTransition;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

import gnu.trove.map.TIntObjectMap;
import gnu.trove.map.hash.THashMap;
import gnu.trove.map.hash.TIntObjectHashMap;
import gnu.trove.procedure.TObjectIntProcedure;

/**
 * Class to keep track of the three transition types used in stochastic trace
 * alignments: - index - transition from the Petri net (TimedTransition) -
 * transition from the alignment (StochasticTransition)
 * 
 * @author sander
 *
 */
public class TransitionMap {

	private final TIntObjectMap<StochasticTransition> index2StochasticTransition = new TIntObjectHashMap<>(10, 0.5f,
			-1);
	private final TIntObjectMap<Transition> index2Transition = new TIntObjectHashMap<>(10, 0.5f, -1);
	private final Map<StochasticTransition, TimedTransition> transition2stochasticTransition = new THashMap<>();

	public TransitionMap(StochasticNet net, EfficientStochasticPetriNetSemanticsImpl semantics) {
		semantics.getTransitionMap().forEachEntry(new TObjectIntProcedure<Transition>() {
			public boolean execute(final Transition transition, final int index) {

				StochasticTransition stochasticTransition = new StochasticTransition() {
					public boolean isInvisible() {
						return transition.isInvisible();
					}

					public String getLabel() {
						return transition.getLabel();
					}

					public int getIndex() {
						return index;
					}
				};

				index2StochasticTransition.put(index, stochasticTransition);
				index2Transition.put(index, transition);
				transition2stochasticTransition.put(stochasticTransition, (TimedTransition) transition);
				return true;
			}
		});
	}

	public StochasticTransition getStochasticTransition(int index) {
		return index2StochasticTransition.get(index);
	}

	public int getNumberOfTransitions() {
		return index2StochasticTransition.size();
	}

	public Transition getTransition(int index) {
		return index2Transition.get(index);
	}
}
