package org.processmining.earthmoversstochasticconformancechecking.parameters;

public class TerminationStrategyImpl implements TerminationStrategy {

	private long startTime;

	private long maxDuration;
	private double maxMassCovered;

	private static final double epsilon = 0.0000000001;

	public TerminationStrategyImpl(long maxDuration, double maxMassCovered) {
		this.setMaxDuration(maxDuration);
		this.setMaxMassCovered(maxMassCovered);
		initialise();
	}

	public void initialise() {
		startTime = System.currentTimeMillis();
	}

	public boolean isTerminated(double massCovered) {
		if (massCovered + epsilon >= getMaxMassCovered()) {
			return true;
		}
		if (System.currentTimeMillis() - startTime > getMaxDuration()) {
			return true;
		}
		return false;
	}

	public TerminationStrategy clone() {
		TerminationStrategyImpl result;
		try {
			result = (TerminationStrategyImpl) super.clone();
			result.setMaxDuration(maxDuration);
			result.setMaxMassCovered(maxMassCovered);
			result.startTime = startTime;
			return result;
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			assert (false);
			return null;
		}

	}

	public double getMaxMassCovered() {
		return maxMassCovered;
	}

	public void setMaxMassCovered(double maxMassCovered) {
		this.maxMassCovered = maxMassCovered;
	}

	public long getMaxDuration() {
		return maxDuration;
	}

	public void setMaxDuration(long maxDuration) {
		this.maxDuration = maxDuration;
	}

}
