package org.processmining.earthmoversstochasticconformancechecking.reallocationmatrix.epsa;

import org.processmining.earthmoversstochasticconformancechecking.distancematrix.DistanceMatrix;
import org.processmining.earthmoversstochasticconformancechecking.parameters.EMSCParameters;
import org.processmining.earthmoversstochasticconformancechecking.plugins.EarthMoversStochasticConformancePlugin;
import org.processmining.earthmoversstochasticconformancechecking.reallocationmatrix.ReallocationMatrix;
import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.StochasticLanguage;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.plugins.InductiveMiner.Pair;

public class ComputeReallocationMatrix2 {
	public static Pair<ReallocationMatrix, Double> compute(StochasticLanguage languageA, StochasticLanguage languageB,
			DistanceMatrix distanceMatrix, EMSCParameters parameters, ProMCanceller canceller)
			throws InterruptedException {

		EarthMoversStochasticConformancePlugin.debug(parameters, "Create distance matrix..");

		distanceMatrix = distanceMatrix.clone();
		distanceMatrix.init(languageA, languageB, canceller);

		if (canceller.isCancelled()) {
			return null;
		}

		EarthMoversStochasticConformancePlugin.debug(parameters, "Create solver..");

		AKPFactoryArraySimple fac = new AKPFactoryArraySimple(distanceMatrix);
		fac.setupNewSolver(languageA, languageB, canceller);

		if (canceller.isCancelled()) {
			return null;
		}

		EarthMoversStochasticConformancePlugin.debug(parameters, "Solve..");

		AKPSolverArraySimpleImprov solver = fac.getSolver();

		double similarity = 1 - solver.solve(canceller);

		if (canceller.isCancelled()) {
			return null;
		}

		EarthMoversStochasticConformancePlugin.debug(parameters, "Create reallocation matrix..");

		ReallocationMatrix relocationMatrix = new ReallocationMatrix(languageA.size(), languageB.size());
		solver.fillMatrix(relocationMatrix);

		//solver.printFinalFlow();

		return Pair.of(relocationMatrix, similarity);
	}
}
