package org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.model;

import java.util.ArrayList;
import java.util.List;

import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.Activity2IndexKey;

import gnu.trove.list.TDoubleList;
import gnu.trove.list.array.TDoubleArrayList;

/**
 * A stochastic path is a list of transitions. (int[]). A
 * StochasticTransition2IndexKey transforms this into a trace or an
 * integerTrace.
 * 
 * @author sander
 *
 */
public class StochasticPathLanguageImpl implements StochasticLanguageModel {

	protected List<int[]> paths = new ArrayList<>();
	protected TDoubleList probabilities = new TDoubleArrayList();
	protected final StochasticTransition2IndexKey transitionKey;
	protected final Activity2IndexKey activityKey;

	public StochasticPathLanguageImpl(StochasticTransition2IndexKey transitionKey, Activity2IndexKey activityKey) {
		this.transitionKey = transitionKey;
		this.activityKey = activityKey;
	}

	/**
	 * Unfortunately, adding to a list is not thread safe.
	 * 
	 * @param path
	 * @param probability
	 */
	public synchronized void add(int[] path, double probability) {
		paths.add(path);
		probabilities.add(probability);
	}

	public StochasticPathIterator iterator() {
		return new StochasticPathIterator() {
			int pathIndex = -1;

			public String[] next() {
				int[] path = nextPath();
				int[] trace = transitionKey.path2trace(path);
				return activityKey.toTraceString(trace);
			}

			public int[] nextPath() {
				pathIndex++;
				return getPath();
			}

			public boolean hasNext() {
				return pathIndex < paths.size() - 1;
			}

			public double getProbability() {
				return probabilities.get(pathIndex);
			}

			public int[] getPath() {
				return paths.get(pathIndex);
			}

			public int[] nextIntegerTrace() {
				pathIndex++;
				return transitionKey.path2trace(getPath());
			}
		};
	}

	public int size() {
		return paths.size();
	}

	public int[] getPath(int pathIndex) {
		return paths.get(pathIndex);
	}

	public String[] getTraceString(int traceIndex) {
		return activityKey.toTraceString(getTrace(traceIndex));
	}

	public int[] getTrace(int traceIndex) {
		return transitionKey.path2trace(getPath(traceIndex));
	}

	public Activity2IndexKey getActivityKey() {
		return activityKey;
	}

	public StochasticTransition2IndexKey getTransitionKey() {
		return transitionKey;
	}
}
