package org.processmining.earthmoversstochasticconformancechecking.tracealignments;

import org.processmining.earthmoversstochasticconformancechecking.helperclasses.TransitionMap;
import org.processmining.earthmoversstochasticconformancechecking.reallocationmatrix.ReallocationMatrix;
import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.model.StochasticLanguageModel;
import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.model.StochasticPath;
import org.processmining.earthmoversstochasticconformancechecking.stochasticlanguage.model.StochasticTransition;
import org.processmining.models.graphbased.directed.petrinet.StochasticNet;

/**
 * Wrapper class as ProM does not support generics.
 * 
 * @author sander
 *
 */
public class StochasticTraceAlignmentsModelModel
		extends StochasticTraceAlignmentsAbstract<StochasticTransition, StochasticTransition> {

	private final StochasticNet netA;
	private final TransitionMap transitionMapA;

	private final StochasticNet netB;
	private final TransitionMap transitionMapB;

	private final StochasticLanguageModel languageA;
	private final StochasticLanguageModel languageB;

	public StochasticTraceAlignmentsModelModel(ReallocationMatrix relocationMatrix, double similarity,
			StochasticNet netA, TransitionMap transitionMapA, StochasticNet netB, TransitionMap transitionMapB,
			StochasticLanguageModel languageA, StochasticLanguageModel languageB) {
		super(relocationMatrix, similarity, languageA, languageB);
		this.languageA = languageA;
		this.languageB = languageB;
		this.netA = netB;
		this.transitionMapA = transitionMapA;
		this.netB = netB;
		this.transitionMapB = transitionMapB;
	}

	@Override
	public EditIteratorModelModel iterator() {
		return new EditIteratorModelModel();
	}

	public class EditIteratorModelModel extends EditIterator {
		public StochasticPath getStochasticPathA() {
			int[] path = languageA.getPath(now);
			return languageA.getTransitionKey().getStochasticPath(path);
		}

		public StochasticPath getStochasticPathB() {
			int[] path = languageB.getPath(now);
			return languageB.getTransitionKey().getStochasticPath(path);
		}
	}

	public StochasticNet getStochasticNetA() {
		return netA;
	}

	public TransitionMap getTransitionMapA() {
		return transitionMapA;
	}

	public StochasticNet getStochasticNetB() {
		return netB;
	}

	public TransitionMap getTransitionMapB() {
		return transitionMapB;
	}
}