package org.processmining.earthmoversstochasticconformancechecking.visualisation;

import java.awt.Color;
import java.util.Map;

import org.processmining.earthmoversstochasticconformancechecking.tracealignments.projection.StochasticTraceAlignmentsModelProjection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.StochasticNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.graphviz.colourMaps.ColourMap;
import org.processmining.plugins.graphviz.colourMaps.ColourMapFixed;
import org.processmining.plugins.graphviz.colourMaps.ColourMapOpacity;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.Dot.GraphDirection;
import org.processmining.plugins.graphviz.dot.DotNode;

import gnu.trove.map.hash.THashMap;

public class StochasticTraceAlignmentsModelVisualisation {

	private static final ColourMap colourMap = new ColourMapOpacity(new ColourMapFixed(Color.red));

	public static Dot visualise(StochasticNet net, StochasticTraceAlignmentsModelProjection projection) {
		Dot dot = new Dot();
		dot.setDirection(GraphDirection.leftRight);
		dot.setOption("forcelabels", "true");

		Map<PetrinetNode, DotNode> place2node = new THashMap<>();

		for (Place place : net.getPlaces()) {
			DotNode dotNode = dot.addNode("");
			dotNode.setOption("shape", "circle");
			place2node.put(place, dotNode);
		}

		for (Transition transition : net.getTransitions()) {
			DotNode dotNode = dot.addNode("");
			dotNode.setOption("style", "filled");
			if (transition.isInvisible()) {
				dotNode.setOption("fillcolor", "#EEEEEE");
				dotNode.setOption("width", "0.15");
			} else {
				double syncProbability = projection.getTransitionSyncLikelihood(transition);
				String syncLabel;
				if (syncProbability < 1) {
					syncLabel = StochasticTraceAlignmentsLogVisualisationTraceComponent.formatNumber(syncProbability)
							+ " sync move";
				} else {
					syncLabel = "1 sync move";
				}

				//dotNode.setOption("xlabel", syncLabel);
				dotNode.setLabel(transition.getLabel() + "\n" + syncLabel);
				dotNode.setOption("fillcolor", colourMap.toHexAlphaString(colourMap.colour(1 - syncProbability)));
			}
			dotNode.setOption("shape", "box");
			place2node.put(transition, dotNode);
		}

		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : net.getEdges()) {
			dot.addEdge(place2node.get(edge.getSource()), place2node.get(edge.getTarget()));
		}

		return dot;
	}
}
